__author__ = 'alexitx'
__author_email__ = 'alexander@alexitx.com'
__license__ = 'GPLv3+'
__title__ = 'eternal-radio-player'
__url__ = 'https://github.com/alexitx/eternal-radio-player'
__version__ = '0.1.1'
